<?php
namespace common\models\search;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\Pagination;
use common\models\Document as DocumentModel;

/**
 * @author vk
 */
class Document extends DocumentModel {

	const STATUS_GROUP_UNDELIVERED = 1;
	const STATUS_GROUP_DELIVERED = 2;
	const STATUS_GROUP_INVALID = 3;
	const STATUS_GROUP_VALID = 4;

	public $id;
	public $typeCode;
	public $absId;
	public $uuid;
	public $signAlgo;
	public $dateIn;
	public $dateInFrom;
	public $dateInBefore;
	public $dateOut;
	public $status;
	public $statusGroup = [
		self::STATUS_GROUP_UNDELIVERED,
		self::STATUS_GROUP_DELIVERED,
		self::STATUS_GROUP_INVALID,
		self::STATUS_GROUP_VALID
	];
	public $statusInner;
	public $errorId;
	public $sender;
	public $rcpt;
	public $ref;
	public $path;
	/**
	 * @var bool
	 */
	public $showTechnical = true;

	public $pageSize = 50;

	public function rules() {
		return [
			[
				['id', 'dateIn', 'dateOut', 'status', 'statusInner', 'errorId', 'uuid', 'sender', 'rcpt', 'typeCode', 'dateInFrom', 'dateInBefore'],
				'filter', 'filter' => 'trim'
			],
			[['id', 'dateIn', 'dateOut', 'status', 'statusInner', 'errorId'], 'integer', 'integerOnly' => true],
			['showTechnical', 'boolean'],
			[['uuid', 'sender', 'rcpt'], 'string', 'length' => [2, 64]],
			['typeCode', 'string', 'length' => [2, 16]],
			['statusGroup','validateStatusGroup'],
			[['dateInFrom', 'dateInBefore'], 'date', 'format' => 'yyyy-MM-dd'],
			[['status'], 'in', 'range' => [DocumentModel::getStatusValues()]],
		];
	}


	public function attributeLabels() {
		return array_merge(
			parent::attributeLabels(),[
				'statusGroup'	=> 'Отображать документы',
				'showTechnical'	=> 'Показывать системные сообщения',
			]
		);
	}


	public function getStatusGroupSetup() {
		return [
			self::STATUS_GROUP_UNDELIVERED => [
				'label'	=> 'Исходящие, не доставленные адресату',
				'value'	=> [
					DocumentModel::STATUS_NOT_READY,
					DocumentModel::STATUS_PICKUP,
					DocumentModel::STATUS_SENT,
					DocumentModel::STATUS_RECEIVED,
					DocumentModel::STATUS_REJECTED
				],
			],
			self::STATUS_GROUP_DELIVERED => [
				'label'	=> 'Исходящие, доставленные адресату',
				'value'	=> [DocumentModel::STATUS_DELIVERED],
			],
			
			self::STATUS_GROUP_VALID => [
				'label'	=> 'Входящие, с валидными подписями',
				'value'	=> [
					DocumentModel::STATUS_EXPORTED,
					DocumentModel::STATUS_SIGNED
				],
			],
			self::STATUS_GROUP_INVALID => [
				'label' => 'Входящие, с невалидными подписями',
				'value' => [DocumentModel::STATUS_SIGNED_INVALID],
			],
		];
	}

	public function getStatusGroupLabels() {
		$t = $this->getStatusGroupSetup();
		$res = [];
		foreach($t as $k => $v) {
			$res[$k] = $v['label'];
		}
		return $res;
	}


	public function validateStatusGroup($attribute, $params) {
		$setup	= $this->getStatusGroupSetup();
		$diff	= array_diff($this->$attribute, array_keys($setup));
		if($this->$attribute && !empty($diff)) {
			$this->addError($attribute);
		}
	}


	public function search($params) {
		$query = DocumentModel::find();

		
		/**
		 * Создаём DataProvider, указываем ему запрос, настраиваем пагинацию
		 */
		$dataProvider = new ActiveDataProvider([
			'query'			=> $query,
			'sort'			=> ['defaultOrder' => ['dateIn'=>SORT_DESC]],
			'pagination'	=> new Pagination([
				'pageSize'	=> $this->pageSize,
			])
		]);

		$this->load($params);
		if (!$this->validate()) {
			$query->andOnCondition('0=1'); // портим выдачу, если были ошибки
			return $dataProvider;
		}

		/**
		 * Интепретируем группы статусов
		 */
		if($this->statusGroup) {
			$this->status = [];
			$setup = $this->getStatusGroupSetup();
			foreach ($this->statusGroup as $v) {
				$this->status = array_merge($this->status,$setup[$v]['value']);
			}
		}
		/**
		 * Разбираемся с показываемыми документами
		 */
		if ($this->showTechnical && $this->status) {
			$query->andFilterWhere([
				'or',
				['in', 'status', $this->status],
				['=', 'typeCode', 'cyberplat/ack']
			]);
		} elseif (!$this->showTechnical && $this->status) {
			$query->andFilterWhere([
				'status' => $this->status,
			]);
			$query->andFilterWhere(['!=', 'typeCode', 'cyberplat/ack']);
		} elseif ($this->showTechnical) {
			$query->andFilterWhere(['=', 'typeCode', 'cyberplat/ack']);
		} elseif (!$this->status) {
			/**
			 * CYB-100 убиваем выдачу если не выбраны типы документов и служебки
			 */
			$query->andOnCondition('0=1');
			return $dataProvider;
		}

		/**
		 * Обработаем фильтр по датам
		 */
		if ($this->dateInFrom) {
			$dFrom = (new \DateTime($this->dateInFrom, new \DateTimeZone(Yii::$app->getTimeZone())))
				->setTimezone(new \DateTimeZone('UTC'))
				->format('Y-m-d H:i:s');

			$query->andFilterWhere(['>=', 'dateIn', $this->dateInFrom ? $dFrom : null]);
		}
		if ($this->dateInBefore) {
			$dBefore = (new \DateTime($this->dateInBefore, new \DateTimeZone(Yii::$app->getTimeZone())))
				->setTimezone(new \DateTimeZone('UTC'))
				->modify('+1 day')
				->format('Y-m-d H:i:s');

			$query->andFilterWhere(['<=', 'dateIn', $this->dateInBefore ? $dBefore : null]);
		}


		$query->andFilterWhere([
			'id' => $this->id,
			'dateIn' => $this->dateIn,
			'dateOut' => $this->dateOut,
			//'statusInner' => $this->statusInner,
			'errorId' => $this->errorId,
			'typeCode' => $this->typeCode,
		])
			->andFilterWhere(['like', 'absId', $this->absId])
			->andFilterWhere(['like', 'uuid', $this->uuid])
			->andFilterWhere(['like', 'signAlgo', $this->signAlgo])
			->andFilterWhere(['like', 'sender', $this->sender])
			->andFilterWhere(['like', 'rcpt', $this->rcpt])
			->andFilterWhere(['like', 'ref', $this->ref])
			->andFilterWhere(['like', 'path', $this->path]);

		
		return $dataProvider;
	}
}
