<?php

namespace common\modules\certManager;

/**
 * @todo ssl = Secure Sockets Layer, переименовать и отрефакторить, режет по мозгу
 * @todo инкапсулировать логику вынесенную в код класса модуля внутрь компонентов, моделей, контроллеров модуля
 */
use common\modules\certManager\components\ssl\Component;
use common\modules\certManager\components\ssl\Exception as SSLException;
use common\modules\certManager\models\Cert;
use Yii;
use yii\base\Module as BaseModule;
use yii\base\Exception;

/**
 * @todo переименовать символьный certId в fullCertId чтобы не путать с int certId модели Cert
 * @todo подумать над переносом сюда механизмов подписи
 * @package common\modules\certManager
 */
class Module extends BaseModule
{
	const CERT_STORAGE_PATH = '@certStorage';
	const KEY_STORAGE_PATH = '@keyStorage';

	protected $error;

    /**
     * @var Component
     */
	public $ssl;
    public $terminalId = 'testCertModule';

	/**
     * Генерация идентификатора сертификата {terminalId}-{fingerprint}
	 * @param string $fingerprint
	 * @return string
	 */
	public static function getCertId($fingerprint) {
        return Yii::$app->terminal->address . '-' . $fingerprint;
	}

	public function init()
    {
        parent::init();

		$this->ssl = Yii::createObject([
			'class' => Component::className(),
			'defaultKeyType'	=> 'RSA',
			'defaultHashAlgo'	=> 'SHA1',
			'keyStoragePath'	=> '@keyStorage',
			'defaultKeyName'	=> 'autobot'
		]);
    }

	/**
	 * Задаем идентификатор терминала на котором работаем
	 * @param string $value
	 */
	public function setTerminalId($value) {
		$this->terminalId = $value;
	}

	public function getError()
	{
		return $this->error;
	}


	/**
	 * Возвращает данные сертификата
     * @param type $certId - идентификатор вида "{terminalId}-{fingerprint}" или id
	 * @return mixed данные сертификата
	 */

	public function getCertificate($certId)
	{
		$cert = Cert::findByCertId($certId);

		return $cert;
	}


	/**
	 * Сохраняет данные сертификата
     * @param type $data
	 * @return string certId
	 */

	public function storeCertificate($params)
    {
		if (empty($params['id']) || empty($params['certData']) || empty($params['terminalId']) ) {
			$this->setError('Missed required fields');
			return false;
		}

        $driver = $this->ssl->getDriver();
		$certDetails = $driver->getCertificateData($params['certData']);
		$params['fingerprint'] = $driver->getCertificateFingerprint($params['certData']);
		$params['validFrom'] = $certDetails['validFrom_time_t'];
		$params['validBefore'] = $certDetails['validFrom_time_t'];

		$attr = [
            'validFrom',
            'validBefore',
            'useBefore',
            'terminalId',
            'fingerprint',
            'status',
            'path',
            'country',
            'state',
            'locality',
            'organization',
            'organizationalUnit',
            'commonName',
            'email',
            'name'
		];
		$args = array_intersect_key($params, array_flip($attr));

		$cert = Cert::findByCertId($params['id']);
		if (!$cert) {
			$cert = new Cert();
		}

        $cert->setAttributes($args);
		if ($cert->validate()) {
			$cert->save();
			$cert->setCertificateContent($params['certData']);

            return true;
		} else {

            return $cert->errors;
		}
	}

    /**
     *
	 * @param type $filter
	 * @param type $page
	 * @param type $pageSize
	 * @return \yii\data\ActiveDataProvider
	 */

    public function getCertificatesList($filter = array(), $page = 0, $pageSize = 10)
	{
		$condition = null;

        $dataProvider = new \yii\data\ActiveDataProvider([
			'query' => Cert::find(),
			'pagination' => [
				'page'	=> $page,
				'pageSize' => $pageSize,
			],
		]);

		return $dataProvider;
	}

    /**
	 * Генерирует набор файлов с ключами и самоподписным сертификатом в `keyStorage`
	 * @param type $filename
	 * @param type $privateKeyPassword
	 * @param type $certData
	 * @param type $driver
	 * @return boolean
	 */
    public function generateKeys($filename, $privateKeyPassword = null, $certData = array(), $driver = null)
	{
		try {
			$result = [];

            $driver = $this->ssl->getDriver((empty($driver) ? Component::DEFAULT_DRIVER : $driver));
			$keys = $driver->generateKeys($privateKeyPassword);
			$result['private'] = $this->storeData($this->getKeyStoragePath(), $keys['private'], $filename, 'private');
			$result['public'] = $this->storeData($this->getKeyStoragePath(), $keys['public'], $filename, 'public');

            $csr = $driver->generateCertificateRequest($certData, $keys['private'], $privateKeyPassword);
			$cert = $driver->signCertificateRequest($csr, null, $keys['private'], $privateKeyPassword);
			$result['cert'] = $this->storeData($this->getKeyStoragePath(), $cert, $filename, 'cert');

            return $result;

        } catch (SSLException $ex) {

            $this->setError($ex->getMessage(), $ex->getCode());

            return false;
        }
	}

    public function getKeyData($filename, $type)
	{
		$type = $this->keyFileTypeExtension($type);

        if ($type) {
			$filePath = $this->getKeyStoragePath() . '/' . $filename . '.' . $type;
			if (
				!file_exists($filePath)
				|| !is_readable($filePath)
			) {
				throw new Exception("Can't access key file '{$filePath}'");
			}
			
			return file_get_contents($this->getKeyStoragePath() . '/' . $filename . '.' . $type );
		}

        return false;
	}

    protected function keyFileTypeExtension($format)
	{
		$type = false;

        switch ($format) {
			case 'private':
				$type = 'key';
				break;
			case 'public':
				$type = 'pub';
				break;
			case 'pkcs':
				$type = 'p12';
				break;
			case 'csr':
				$type = 'csr';
				break;
			case 'cert':
				$type = 'crt';
				break;
		}

        return $type;
	}

	protected function storeData($storagePath, $data, $filename, $type)
	{
		$type = $this->keyFileFormatExtension($type);

        if (!$type) {
			throw new Exception('Unknown file type');
		}

        $filePath = $storagePath . '/' . $filename . '.' . $type;
		file_put_contents($filePath, $data);

        return $filePath;
	}

    /**
     *
	 * @param type $fingeprint
	 * @return string
	 */
	protected function getCertificateDriver($fingeprint)
	{
		/**
		 * @todo Определить какой SSL driver по данным сертификата
		 */
        return Component::DEFAULT_DRIVER;
	}

    protected function setError($message, $code = 0)
    {
		$this->error = [
			'message'	=> $message,
			'code'		=> $code
		];
	}

    public static function getCertStoragePath()
	{
		return Yii::getAlias(self::CERT_STORAGE_PATH);
	}

    public static function getKeyStoragePath()
	{
		return Yii::getAlias(self::KEY_STORAGE_PATH);
	}

}