<?php
namespace common\modules\certManager\components\ssl;

use common\modules\certManager\components\ssl\Exception as SSLException;
use common\modules\certManager\models\Cert;
use Yii;
use yii\base\Component as BaseComponent;

class Component extends \yii\base\Component {
	
	const DEFAULT_DRIVER = 'OpenSSL';
	
	public $keyStoragePath;
	public $defaultKeyName;
	public $defaultKeyType;
	public $defaultHashAlgo;
	
	public $keyPrefix = '';
	
    protected $driverMap = [
        'OpenSSL'   => 'DriverOpenSSL',
        'CryptoPro'   => 'DriverCryptoPro',
    ];

	
	public function init()
	{
		parent::init();
		$this->keyStoragePath = Yii::getAlias($this->keyStoragePath);
		if (!is_dir($this->keyStoragePath) || !is_writable($this->keyStoragePath)) {
			throw new Exception('Invalid key storage path "'.$this->keyStoragePath.'"');
		}
	}
	
    public function getDriver($driver=self::DEFAULT_DRIVER, $keyType = '', $hashAlgo = '') 
	{
		if (!$keyType) {
			$keyType = $this->defaultKeyType;
		}
		
		if (!$hashAlgo) {
			$hashAlgo = $this->defaultHashAlgo;
		}
		
		$className = __NAMESPACE__  . '\\' . $this->driverMap[$driver];

		return new $className([
				'keyType'	=> $keyType,
				'hashAlgo'	=> $hashAlgo,
			]);
    }
	
	
	/**
	 * Подписать данные
	 * @param type $data
	 * @param type $privateKey
	 * @param type $password
	 * @return string подпись в PEM формате
	 */
	public function sign($data, $privateKey, $password = null, $driver = self::DEFAULT_DRIVER) 
	{
		$driver = $this->getDriver($driver);
		$signature = $driver->sign($data, $privateKey, $password);
		
		return $signature;
	}
	
	
	/**
	 * Верификация подписи
	 * 
	 * @param type $data - данные
	 * @param type $signature - цифровая подпись
	 * @param type $certFullId - полный идентификатор сертификата
	 * @return boolean true если подпись верна
	 */
	public function verify($data, $signature, $certFullId, $driver = self::DEFAULT_DRIVER)
	{
        $certContent = $this->getCertContent($certFullId);
		return $this
			->getDriver($driver)
			->verify($data, $signature, $certContent);
	}
	
	
	public function signPkcs7() {
		throw new Exception('Not implemented yet');
	}
	
	/**
	 * 
	 * @param string $data - unsigned raw data
	 * @param string $signature - binnary string with container
	 * @param string $certFullId - полный идентификатор сертификата
	 * @param type $isDetached
	 * @param type $driver
	 * @return boolean
	 */
	public function verifyPkcs7($data, $signature, $certFullId, $isDetached, $driver = self::DEFAULT_DRIVER)
	{
        $certPath = $this->getCertPath($certFullId);
		return $this
				->getDriver($driver)
            ->verifyPkcs7($data, $signature, $certPath, $isDetached);
	}
	
		
	public function generateKeyFilename()
	{
		return uniqid();
	}
	
	protected function getKeyFilename()
	{

    }

    /**
     * Получение тела сертификата
     * @param string $certFullId
     * @return string
     * @throws Exception
     */
    protected function getCertContent($certFullId)
    {
        if (!($certContent = $this->getCert($certFullId)->getCertificateContent())) {
            throw new Exception('Can\'t get certificate file', Exception::ERROR_BROKEN_CERT_PATH);
        }
        return $certContent;
    }

    /**
     * Получение пути до файла с сертификатом
     * @param string $certFullId
     * @return string
     */
    protected function getCertPath($certFullId)
    {
        return $this->getCert($certFullId)->getFilePath();
    }

    /**
     * Получение тела сертификата
     * @param string $certFullId
     * @return string
     * @throws Exception
     */
    protected function getCert($certFullId)
    {
		$cert = Cert::findByCertId($certFullId);
		if (empty($cert)) {
            throw new Exception('Unknown certificate', Exception::ERROR_UNKNOWN_CERT_ID);
        }
        return $cert;
	}
}
