<?php
namespace common\modules\certManager\components\ssl;

use common\modules\certManager\components\ssl\Driver;

class DriverOpenSSL extends Driver
{	
	
	protected $keyTypesMap = [
		'RSA'	=> OPENSSL_KEYTYPE_RSA,
		'DSA'	=> OPENSSL_KEYTYPE_DSA,
		'DH'	=> OPENSSL_KEYTYPE_DH,
	];
	
    protected $cipherAlgoMap = [
        'RC2-40'	=> OPENSSL_CIPHER_RC2_40,
		'RC2-128'	=> OPENSSL_CIPHER_RC2_128,
		'RC2-64'	=> OPENSSL_CIPHER_RC2_64,
		'DES'		=> OPENSSL_CIPHER_DES,
		'3DES'		=> OPENSSL_CIPHER_3DES,
		'AES-128'	=> OPENSSL_CIPHER_AES_128_CBC,
		'AES-192'	=> OPENSSL_CIPHER_AES_192_CBC,
		'AES-256'	=> OPENSSL_CIPHER_AES_256_CBC,
    ];

	protected $signAlgoMap = [
		'DSS1'	=> OPENSSL_ALGO_DSS1,
		'SHA1'	=> OPENSSL_ALGO_SHA1,
		'MD5'	=> OPENSSL_ALGO_MD5,
		'MD4'	=> OPENSSL_ALGO_MD4,
		//'MD2'	=> OPENSSL_ALGO_MD2,
	];

	
	
	/**
	 * Генерация пары ключей
	 * 
	 * @param type $password 
	 * @param type $size длина приватной части ключа
	 * @return type
	 */
	
	public function generateKeys($password = null, $size = self::KEY_SIZE)
	{
		$config = [
			'digest_alg' => 'sha1',
			'private_key_bits' => $size, // Size of Key.
			'private_key_type' => OPENSSL_KEYTYPE_RSA,
		];
		
		$res = openssl_pkey_new();		
		openssl_pkey_export($res, $privateKey, $password);
		
		// Generate the public key for the private key
		$privateKeyDetails = openssl_pkey_get_details($res);

		// Free the private Key.
		openssl_free_key($res);
		
		return [
			'private'	=> $privateKey,
			'public'	=> $privateKeyDetails['key']
		];
	}
	
	public function openPrivateKey($pem, $password = null) 
	{
		$pKey = openssl_pkey_get_private($this->fixPemFilePath($pem), $password);
		
		if (!$pKey) {
			throw new Exception('Cannot open private key. OpenSSL error: "' . openssl_error_string().'"', self::ERROR_PKEY_INVALID);
		}
		
		return $pKey;
	}
	
	/**
	 * Генерация сертификата на основе приватного ключа
	 * 
	 * @param array $dn certificate data
	 * @param string $privateKey PEM formatted filename or string
	 * @param string $password
	 * @return string PEM
	 */
	public function generateCertificateRequest($dn, $privateKey, $password = null) {
		
		$pKey = $this->openPrivateKey($privateKey, $password);
		
		// Generate a certificate signing request
		$csr = openssl_csr_new($dn, $pKey);
		
		openssl_csr_export($csr, $csrout);
		
		openssl_free_key($pKey);
	
		return $csrout;
	}
	
	/**
	 * Подписать заявку на сертификат
	 * 
	 * @param type $csr
	 * @param type $cacert
	 * @param type $privateKey PEM format
	 * @param type $password
	 * @param type $days
	 * @param type $optional
	 * @return type signed certificate PEM
	 */
	
	public function signCertificateRequest($csr, $cacert, $privateKey, $password = null, $days = self::CERT_DAYS, $optional = null)
	{
		$pKey = $this->openPrivateKey($privateKey, $password);
		$cert = openssl_csr_sign($this->fixPemFilePath($csr), $cacert, $pKey, $days);
		openssl_x509_export($cert, $certout);
	
		openssl_free_key($pKey);
		
		return $certout;
	}
	
	/**
	 * 
	 * @param type $dn
	 * @param type $days
	 * @param type $privateKey
	 * @param type $password
	 * @return type
	 */
	
	public function generateSelfSignedCertificate($dn, $days, $privateKey, $password) 
	{
		
		$csr = $this->generateCertificateRequest($dn, $privateKey, $password);
		
		$cert = $this->signCertificateRequest($csr, null, $privateKey, $password, $days);
		
		return $cert;
	}
	
	
	
	/**
	 * 
	 * @param type $cert PEM or file
	 */
	
	public function getCertificateData($cert) {
		return openssl_x509_parse($this->fixPemFilePath($cert), true);
	}
	
	/**
	 * 
	 * @param type $cert PEM or file
	 * @return type
	 */
	public function getCertificateFingerprint($cert) {
		$resource = openssl_x509_read($this->fixPemFilePath($cert));

		$fingerprint = null;
		$output = null;

		$result = openssl_x509_export($resource, $output);
		if($result !== false) {
			$output = str_replace('-----BEGIN CERTIFICATE-----', '', $output);
			$output = str_replace('-----END CERTIFICATE-----', '', $output);

			$output = base64_decode($output);

			$fingerprint = strtoupper(sha1($output));
		}
		
		return $fingerprint;
	}
	
	/**
	 * 
	 * @param type $cert
	 * @return type
	 */
	
	public function getCertificatePublic ($cert ) 
	{
		return openssl_pkey_get_public($this->fixPemFilePath($cert));
	}
	
	/**
	 * Sign string data
	 * @param string $data
	 * @param type $privateKey
	 * @param type $password
	 * @return type
	 */
	public function sign($data, $privateKey, $password = null)
	{
		$privateKey = $this->openPrivateKey($privateKey, $password);
		
		openssl_sign($data, $signature, $privateKey);
		
		return $signature;
	}
	
	/**
	 * 
	 * @param type $data
	 * @param type $signature
	 * @param type $pub_key
	 * @param type $signature_alg
	 */
    public function verify($data , $signature , $pubKey)
    {
        return openssl_verify($data, $signature, $this->fixPemFilePath($pubKey));
    }
	
	/**
     *
     * @param string|null $data - raw data
     * @param string $signature - base64 encoded binnary string or path to file
     * @param type $certPath
	 * @param type $isDetached
	 * @return type
	 */
    public function verifyPkcs7($data, $signature, $certPath, $isDetached)
    {
        $signature = base64_decode($signature);
        if (substr_count($signature, 'file://')) {
            $file = str_replace('file://', '', $signature);
		} else {
			$isTmpFile = true;
            $t2 = tempnam(sys_get_temp_dir(), 'cybertswift');
            $file = tempnam(sys_get_temp_dir(), 'cybertswift');
            file_put_contents($file, $signature);
		}

        $out = [];
        exec('openssl cms -verify -noverify -in ' . $file . ' -inform DER -signer ' . $certPath . ' 2>&1', $out);
        $success = (isset($out[0]) && 'Verification successful' === $out[0]);
        /**
         * строка приходит в ASCII с 0-м символом после каждого изначально UTF8 символа
         */
        $verifiedData = str_replace(chr(0), null, $out[1]);
		
		if($isTmpFile) {
            @unlink($file, $t2);
		}
        return $success;

//		$flags = PKCS7_NOINTERN|PKCS7_NOCHAIN|PKCS7_NOVERIFY|PKCS7_BINARY;
//		$res = openssl_pkcs7_verify($file, $flags, null, [], $certPath);
//		var_dump(
//			openssl_error_string(),
//			$file.': '.is_file($file),
//			$certPath.': '.is_file($certPath)
//		);
    }
	
	/**
	 * Fix for filepath
	 * @param type $pem
	 */
	protected function fixPemFilePath($pem) {
		if (!$this->isPemString($pem)) { // не PEM, а путь
			$pem = 'file://' . $pem;
		}
		
		return $pem;
	}
	
	protected function isPemString($string) {
		return $string[0] == '-';
	}
}
