<?php

namespace common\modules\certManager\models;

use common\models\search\Document;
use common\models\User;
use common\models\UserHasCert;
use common\modules\certManager\Module as CertManagerModule;
use Yii;
use yii\db\ActiveQuery;
use yii\db\ActiveRecord;


/**
 * This is the model class for table "cert".
 *
 * @property integer $id
 * @property string $validFrom
 * @property string $validBefore
 * @property string $useBefore
 * @property string $terminalId
 * @property string $fingerprint
 * @property string $status
 * @property string $path
 * @property string $country
 * @property string $state
 * @property string $locality
 * @property string $organization
 * @property string $organizationalUnit
 * @property string $commonName
 * @property string $email
 * @property string $name
 *
 * @property CertSignDocument[] $certSignDocuments
 * @property Document[] $documents
 * @property UserHasCert $userHasCert
 * @property User $user
 */
class Cert extends ActiveRecord
{
    const CERT_ID_DIVIDER = '-';
	
	const STATUS_C0 = 'C0'; //Сертификат зарегистрирован в БД Терминала, но не загружен в хранилище;
	const STATUS_C1 = 'C1'; //сертификат установлен;
	const STATUS_C2 = 'C1'; //сертификат установлен вручную ГА;;
	const STATUS_C7 = 'C7'; //сертификат временно заблокирован;
	const STATUS_C8 = 'C8'; //сертификат заблокирован но еще не удален из рабочей папки;
	const STATUS_C9 = 'C9'; //сертификат отозван или заблокирован;
	
	
	/**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'cert';
    }

    protected static function parseCertId($certId)
    {
        return explode(self::CERT_ID_DIVIDER, $certId);
    }

    public static function findByCertId($certId)
    {
        $idData = self::parseCertId($certId);

        $condition = [];

        if (2 == count($idData)) { // передан certId
            $condition = [
                'terminalId' => $idData[0],
                'fingerprint' => $idData[1],
            ];
        } elseif (intval($certId)) { //передан id
            $condition = [
                'id' => $certId
            ];
        }

        if (!$condition) {
            return false;
        }

        $cert = Cert::findOne($condition);

        return $cert;
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['validFrom', 'validBefore', 'useBefore', 'terminalId', 'fingerprint', ], 'required'],
            [['validFrom', 'validBefore', 'useBefore'], 'integer'],
            [['terminalId'], 'string', 'max' => 12],
            [['fingerprint', 'state', 'locality', 'organization', 'organizationalUnit', 'commonName', 'email', 'name'], 'string', 'max' => 64],
            [
				['status'],
				'string',
				'max' => 16,
				'range' => [self::STATUS_C0, self::STATUS_C1, self::STATUS_C2, self::STATUS_C7, self::STATUS_C8, self::STATUS_C9]
			],
            [['path'], 'string', 'max' => 255],
            [['country'], 'string', 'max' => 2],
            [
				['terminalId', 'fingerprint'],
				'unique',
				'targetAttribute' => ['terminalId', 'fingerprint'],
				'message' => 'The combination of Terminal ID and Fingerprint has already been taken.'
			]
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'validFrom' => 'Valid From',
            'validBefore' => 'Valid Before',
            'useBefore' => 'Use Before',
            'terminalId' => 'Terminal ID',
            'fingerprint' => 'Fingerprint',
            'status' => 'Status',
            'path' => 'Path',
            'country' => 'Country',
            'state' => 'State',
            'locality' => 'Locality',
            'organization' => 'Organization',
            'organizationalUnit' => 'Organizational Unit',
            'commonName' => 'Common Name',
            'email' => 'Email',
            'name' => 'Name',
        ];
    }

	public function setStatus($status) 
	{
		
		
	}

	/**
	 * Вернет наиболее раннюю дату, по истечении которой сертификат не валиден
	 * @return string
	 */
	public function getValidBeforeReal()
	{
		$d1 = new \DateTime($this->validBefore);
		$d2 = new \DateTime($this->useBefore);
		if ((new \DateTime($this->validBefore)) <= (new \DateTime($this->useBefore))) {
			return $this->validBefore;
		} else {
			return $this->useBefore;
		}
	}
	
    /**
     * @return ActiveQuery
     */
    public function getCertSignDocuments()
    {
        return $this->hasMany(CertSignDocument::className(), ['certId' => 'id']);
    }

    /**
     * @return ActiveQuery
     */
    public function getDocuments()
    {
        return $this->hasMany(Document::className(), ['id' => 'documentId'])->viaTable('{certSignDocument}', ['certId' => 'id']);
    }

    /**
     * @return ActiveQuery
     */
    public function getUserHasCert()
    {
        return $this->hasOne(UserCert::className(), ['certId' => 'id']);
    }


    /**
     * @return User
     */
    public function getUser()
    {
        return $this->hasMany(User::className(), ['id' => 'userId'])->viaTable('userHasCert', ['certId' => 'id'])->one();
    }
	
	
	/**
     *
     * @return string PEM
	 */
	public function getCertificateContent()
	{
        $path = $this->getFilePath();
        if (file_exists($path)) {
            return file_get_contents($path);
		}

        return false;
	}

	/**
	 * Запись сертификата в файл
	 * @param type $string
	 */
	public function setCertificateContent($string)
	{
        file_put_contents($this->getFilePath(), $string);
    }

    public function getFilePath()
    {
        return CertManagerModule::getCertStoragePath() . '/' . $this->getCertId() . '.' . $this->getFileExtention();
	}
	
	public function getCertId()
	{
		return $this->terminalId . self::CERT_ID_DIVIDER . $this->fingerprint;
	}

    protected function getFileExtention()
    {
        return 'crt';
	}
}
