<?php

namespace common\modules\docManager;

use common\models\Document;
use common\modules\certManager\components\ssl\Driver;
use common\modules\docManager\components\Storage;
use common\modules\docManager\components\SwtParser;
use Yii;
use yii\base\Exception;
use yii\base\Module as BaseModule;

class Module extends BaseModule {

	public $controllerNamespace = 'common\modules\docManager\controllers';

	const UUID_SYSTEM_COMMAND = 'uuid';
	
	const DIR_OUTGOING = 'out';
	const DIR_INCOMING = 'in';
	const DIR_WORKING = 'working';
	const DIR_SAVED = 'saved';
	const DIR_DISTRUSTED = 'working';
	
	const ERROR_FATAL = 500;
	
	const ERROR_DB_ACCESS = 501;
	const ERROR_SOURCE_FILE_ACCESS = 502;
	const ERROR_SOURCE_FILE_FORMAT = 503;
	const ERROR_PRIVATE_KEY_ACCESS = 504;
	const ERROR_ACCESS_STORAGE_DIR = 510;
	const ERROR_ACCESS_WORKING_DIR = 511;
	const ERROR_INVALID_KEY_PASSWORD = 512;
	const ERROR_SSL = 513;
	const ERROR_OUTPUT_FILE = 514;
	
	const ERROR_DOC_STATUS_VIOLATION = 550;
	const ERROR_DOC_NOT_FOUND = 551;
	const ERROR_DOC_VALIDATION = 552;

	public $swtParser;
	public $storage;

	/**
	 * 
	 */
	public function bootstrap($app)
	{
		
	}

	public function init()
	{
		parent::init();

		$this->aliases = [
			'@templates' => '@common/modules/docManager/templates',
			'@tmplEnvelope' => '@templates/envelope.php'
		];

		$this->swtParser = Yii::createObject([
				'class' => SwtParser::className(),
		]);

		$this->storage = Yii::createObject(Storage::className(), ['maxFiles'	=> 10]);

		if (!is_readable($this->getOutgoingPath()) || !is_readable($this->getIncomingPath())) {
			throw new Exception(Yii::t('app/error', 'Cannot access doc storage'), self::ERROR_ACCESS_STORAGE_DIR);
		}

		if (!is_writable($this->getWorkingPath()) || !is_writable($this->getDistrustedPath())) {
			throw new Exception(Yii::t('app/error', 'Cannot write into doc storage'), self::ERROR_ACCESS_WORKING_DIR);
		}
	}

	/**
	 * 
	 * @param type $docFilename
	 */
	public function registerOutgoingDoc($docFilename, $deleteSource = false)
	{
		$docFilepath = $this->getOutgoingPath() . '/' . $docFilename;
		
	
		$uuid = $this->generateUUID();
		
		$storagePath = $this->storage->getActualDir();
		$savedPath = $storagePath . '/' . $uuid . '.'. $docFilename;
		
		if (!$this->isValidSourceFile($docFilepath)) {
			
			throw new Exception(Yii::t('app/error',  'Invalid source file format'), self::ERROR_SOURCE_FILE_FORMAT);
		}
		
		if (
			!file_exists($docFilepath)
			|| !is_readable($docFilepath)
		) {	
			
			throw new Exception(Yii::t('app/error',  'Cannot access source file'), self::ERROR_SOURCE_FILE_ACCESS);
		}
		
		if (!copy($docFilepath, $savedPath)) {
			throw new Exception(Yii::t('app/error', 'Cannot save into storage dir'), self::ERROR_ACCESS_STORAGE_DIR);
		}
		
		chmod($savedPath, 0665);

		if ($deleteSource) {
			if (!unlink($docFilepath)) {
				throw new Exception(Yii::t('app/error', 'Can\'t delete source file'), self::ERROR_SOURCE_FILE_ACCESS);
			}
		}
		
		$doc = new Document();
		$doc->setScenario('register');
		$doc->pathSource = $savedPath;
		
		$doc->setAttributes([
			'sender'		=> $doc->swtDocument->sender,
			'rcpt'			=> $doc->swtDocument->recipient,
			'typeCode'		=> $doc->swtDocument->contentType,
			'dateIn'		=> gmdate('Y-m-d H:i:s'),
			'uuid'			=> $uuid,
			'status'		=> Document::STATUS_NOT_READY,
			'statusInner'	=> Document::STATUS_INNER_REGISTERED,
		]);
		

		/**
		if ($doc->hasErrors()) {
			if ($doc->hasErrors('sender')) {
				$doc->statusInner = Document::STATUS_INNER_SENDER_INVALID;
				$doc->clearErrors('sender');
			}
			$doc->clearErrors();
		}**/
		
		$doc->save();

		if ($doc->hasErrors()) {
			
			throw new Exception("Cannot save file data:\n * ". join("\n * ", $doc->firstErrors) , self::ERROR_SOURCE_FILE_FORMAT);
		}
			
		return $doc;
	}
	
	public function isValidSourceFile($filePath)
	{
		$info = pathinfo($filePath);
		if (!in_array(strtolower($info['extension']), ['swt'])) {
			
			return false;
		}
		
		return true;
	}
	
	/**
	 * 
	 * @param Document $document
	 * @param type $autobot
	 * @return boolean
	 * @throws Exception
	 */
	public function documentSign(Document $document, $autobot = true)
	{
		if (!$autobot) {
			throw new Exception('Not supported', self::ERROR_FATAL);
		}
		
		if (empty($document)) {
			
			throw new Exception(Yii::t('app/error', 'Unknown document'), self::ERROR_DOC_NOT_FOUND);
		}
		
		if (Document::STATUS_INNER_REGISTERED != $document->statusInner) {
			
			throw new Exception(Yii::t('app/error', 'Status must be \'registered\''), self::ERROR_DOC_STATUS_VIOLATION);
		}
		
		$data = $document->swtDocument->data;
		if (empty($data) || !array_key_exists('content', $data)) {
			throw new Exception(Yii::t('app/error', 'Invalid document data'), self::ERROR_SOURCE_FILE_FORMAT);
		}
		
		$signedData = $this->getAutobotSignature($document->swtDocument->content);
		
		if (!$filePath = $this->documentExport($document, $signedData)) {
			throw new Exception(Yii::t('app/error', 'Exporting document failed'), self::ERROR_OUTPUT_FILE);
		}
		
		$document->path = $filePath;
		$document->statusInner = Document::STATUS_INNER_SIGNED;
		
		$document->save();
		
		return true;
	}
	
	/**
	 * Установка адреса терминала в качестве адреса отправителя
	 * 
	 * @param Document $document
	 * @throws BadRequestHttpException
	 */
	public function documentFixSender(Document $document)
	{
		if (Document::STATUS_INNER_SENDER_INVALID != $document->statusInner) {
			throw new Exception(Yii::t('app/error', "Status should be '{status}'", ['status'  => Document::STATUS_INNER_SENDER_INVALID]));
		}
		
		$document->sender = Yii::$app->terminal->address;
		$document->status = Document::STATUS_NOT_READY;
		$document->statusInner = Document::STATUS_INNER_REGISTERED;
		
		$document->swtDocument->sender = $document->sender;
		$document->swtDocument->save();
		
		if (!$document->save()) {
			throw new Exception(Yii::t('app/error', 'Can\'t save document') . ': ' . print_r($document->getErrors(), true), self::ERROR_DOC_VALIDATION);
		}
		
		return true;
	}
	
	
	/**
	 * 
	 * @param Document $document
	 */
	public function documentSend(Document $document)
	{
		$document->status = Document::STATUS_PICKUP;
		if ($document->save()) {
			return true;
		}
		
		return;
	}

	/**
	 * @todo Это должно переехать поближе к CertManager
	 * 
	 * @param type $content
	 * @return type
	 * @throws Exception
	 */
	
	public function getAutobotSignature($content)
	{
		$certManager = Yii::$app->getModule('certManager');
		$autobotConfig = Yii::$app->terminal->config['autobot'];
		
		$procConfig	= Yii::$app->terminal->autoProcessingConfig;
		
		if (empty($procConfig) || !array_key_exists('privatePassword', $procConfig)) {
			throw new Exception(Yii::t('app/error', 'Invalid autoprocessing vars'), self::ERROR_FATAL);
		}
		
		$private = $certManager->getKeyData($autobotConfig['keyname'], 'private');
		
		if (empty($private)) {
			throw new Exception(Yii::t('app/error', 'Private key invalid'), self::ERROR_PRIVATE_KEY_ACCESS);
		}
		
		try {
			
			$signature = $certManager->ssl->sign($content, $private, $procConfig['privatePassword']);
		} catch (Exception $ex) {
			switch ($ex->getCode()) {
				case Driver::ERROR_PKEY_INVALID :
					throw new Exception(Yii::t('app/error', 'Private key invalid password'), self::ERROR_INVALID_KEY_PASSWORD);
					break;
				default:
					throw new Exception(Yii::t('app/error', 'SSL error'), self::ERROR_SSL);
					break;
			}
		}

		return [
			'signature' => $signature,
			'fingerprint' => $autobotConfig['certFingerprint'],
			'content' => $content
		];
	}
	
	protected function documentExport(Document $doc, $data)
	{
		$date = date('c');
		$signature = base64_encode($data['signature']);
		$content = base64_encode($data['content']);

		$envelopeContent = <<<EOT
Msg-ID: {$doc->uuid}
From: {$doc->sender}
To: {$doc->rcpt}
Date: {$date}
Content-Type: {$doc->typeCode}
Approved: {$data['fingerprint']} {$signature}
Begin
{$content}
End
EOT;

		$filePath = Yii::getAlias("@docStorage/working/{$doc->uuid}");
		$result = file_put_contents(
				$filePath, $envelopeContent
		);

		if (false === $result) {
			throw new Exception(Yii::t('app/error', 'Error while writing into envelope'), self::ERROR_OUTPUT_FILE);
		}

		return $filePath;
	}

	public function getWorkingDoc($uuid)
	{
		$result = false;
		$filename = $this->getWorkingPath() . '/' . $uuid;
		if (file_exists($filename)) {
			$result = file_get_contents($filename);
		}

		return $result;
	}

	public function generateUUID()
	{
		return strtoupper(trim(shell_exec(self::UUID_SYSTEM_COMMAND)));
	}

	public function getOutgoingPath()
	{
		return Yii::getAlias("@docStorage/" . self::DIR_OUTGOING);
	}

	public function getIncomingPath()
	{
		return Yii::getAlias("@docStorage/" . self::DIR_INCOMING);
	}

	public function getWorkingPath()
	{
		return Yii::getAlias("@docStorage/" . self::DIR_WORKING);
	}
	
	public function getSavedPath()
	{
		return Yii::getAlias("@docStorage/" . self::DIR_SAVED);
	}

	public function getDistrustedPath()
	{
		return Yii::getAlias("@docStorage/" . self::DIR_DISTRUSTED);
	}

}
