<?php

namespace console\controllers;

use common\models\Document as DocumentModel;
use Yii;
use yii\base\Exception;
use yii\console\Controller;
use yii\helpers\Console;

/**
 * Manages document entities
 */
class Document extends Controller
{
	/**
     * @var boolean if flag is set source file will be deleted after 
	 * copying into storage
     */
	public $deleteSource = false;

	/**
     * @var string Formats: syslog, console
     */
	public $outputFormat = 'console';
	
	/**
	public function init()
	{
		if ('syslog' == $this->outputFormat) {
			openlog('CyberFT',  LOG_PID | LOG_PERROR, LOG_LOCAL0);
		}
	}
	 * 
	 */
	
	/**
	 * 
	 * Help message
	 */
    public function actionIndex()
    {
		 $this->run('/help', ['document']);
    }
	
	public function options($actionID)
	{
		$customOptions = ['outputFormat'];
		if ($actionID == 'load') {
			array_push($customOptions, 'deleteSource');
		}
		
		return array_merge(
            parent::options($actionID),
            $customOptions
        );
	}
	
	/**
	 * Loads MT-document from file
	 * 
	 * @param string $filePath the path or alias of the source MT file.
	 */
	
	public function actionLoad($filePath = '')
	{		
		if (!$filePath) {
			$filePath = $this->prompt('Source file');
		}
		
		$procConfig = Yii::$app->terminal->autoProcessingConfig;
		
		$module = Yii::$app->getModule('docManager');
		try {
			$doc = $module->registerOutgoingDoc($filePath, $this->deleteSource);
			if (empty($doc)) {
				$this->output("Invalid document: {$filePath}");
			} else {
				$this->output("Created document ({$doc->id}) {$doc->uuid} ");
			}
			
			if (0 == $doc->statusInner) {
				Console::output(" * Document error ");				
			}	
			
			
			if ( DocumentModel::STATUS_INNER_SENDER_INVALID == $doc->statusInner) {
				Console::error("Invalid sender");
				if ( !empty($procConfig['fixSender']) ) {
					$module->documentFixSender($doc);
					Console::output("* Fixed");
				}
			}
		} catch (Exception $e) {
			/**
			 * @todo Логирование ошибок
			 */
			Console::error($e->getMessage());
		}
		
	}
	
	/**
	 * Batch sign and send registered documents 
	 * 
	 * @return type
	 */
	public function actionAutoprocess($count = 100)
	{
		$this->output("Autoprocessing started");
		
		try {
			$procConfig = Yii::$app->terminal->autoProcessingConfig;
			
			
			if (empty($procConfig['started']) ) {
				$this->output("Autoprocessing not allowed");
				
				return Controller::EXIT_CODE_NORMAL;
			} 
			
			// Find all registered valid docs
			$documents = DocumentModel::find()->where([
				'status'		=> DocumentModel::STATUS_NOT_READY,
				'statusInner'	=> DocumentModel::STATUS_INNER_REGISTERED,
			])->limit($count)->all();
			
			
			if (empty($documents)) {
				$this->output("Autoprocessing have no data");
				
				return Controller::EXIT_CODE_NORMAL;
			}
			
			foreach ($documents as $doc) {				
				$this->processDoc($doc);
			}
			
		} catch (Exception $e) {
			/**
			 * @todo Логирование ошибок
			 */
			Console::error($e->getMessage());
		}
		
		$this->output("Autoprocessing stopped");
		return Controller::EXIT_CODE_NORMAL;
	}
	
	
	protected function processDoc(DocumentModel $doc)
	{
		$module = Yii::$app->getModule('docManager');
		$message = "Autoprocessing doc ({$doc->id}) {$doc->uuid}";
		
		// автоподписание документа
		if (
			DocumentModel::STATUS_INNER_REGISTERED == $doc->statusInner
			&& DocumentModel::STATUS_NOT_READY == $doc->status
		){

				$module->documentSign($doc);
				$message.= "* Signed";
		}

		// авто отправление
		if (
			DocumentModel::STATUS_INNER_SIGNED == $doc->statusInner
			&& DocumentModel::STATUS_NOT_READY == $doc->status
		){
			$module->documentSend($doc);
			$message.= "* Sent";
		}
		
		$this->output($message);
		return true;
	}
	

	protected function output($message, $priority = 0) 
	{
		if ('syslog' == $this->outputFormat) {
			syslog(LOG_INFO, $message);
		} else {
			Console::output($message);
		}
	
		
	}
	
	
	/**
	 * View document info
	 * @param type $uuid
	 */
	
	public function actionView($uuid)
	{
		$doc = DocumentModel::findOne(['uuid' => $uuid]);
		Console::output("Document data:");
		Console::output(print_r(array_merge(
			$doc->attributes, 
			[
				'contents'	=> file_get_contents($doc->path),
			]
		), true));
		
		Console::output(print_r([
			$doc->swtDocument->sender,
			$doc->swtDocument->recipient,
		], true));
	}
	
	/**
	 * Generates SWT formatted files in `storage/docs/out`
	 * 
	 * @param type $count
	 * @return type
	 */
	public function actionGenerate($count = 10)
	{
		if (!YII_ENV_DEV) {
			Console::error("Command is not supposed to be used in production");
			return Controller::EXIT_CODE_ERROR;
		}
		
		$template = <<<EOT
{1:F01TESTRUM@A0010000000000}{2:I103TESTRUM3A001N}{4:
:20:140409/2674.E
:23B:CRED
:32A:140409EUR11111,1
:33B:EUR11111,1
:50F:/40702978100000001111
1/ZAO IVANCO
2/IVANOVSY PER. D.1 STR.1
3/RU/MOSCOW
:52A:PLATRUMM
:53B:/100 1001011 0000
:57A:COBADEFF550
:59:/DE11110400220200222222
MERSEDES AG
:70:CNTR 55555 DD 01.03.2001.12345678/
2347/0000/2/1 01.01.10 87654321/140
314/0011747 INV2001-10011 07.03.12
FOR SHEETS AND FILTER.
:71A:OUR
-}
EOT;
		$docManager = Yii::$app->getModule('docManager');
		$path = $docManager->getOutgoingPath() . '/';
		
		for ($i = 1; $i<= $count; $i++ ) {
			file_put_contents($path . uniqid() . '.swt', $template);
		}
		
		Console::output('Generated ' . $count . ' files');
	}
	
	/**
	 * Cleans up documents table 
	 * 
	 */
	
	public function actionPurge() 
	{
		if (!YII_ENV_DEV) {
			Console::error("Command is not supposed to be used in production");
			return Controller::EXIT_CODE_ERROR;
		}

		if ($this->confirm('Are you sure?') && $this->confirm('Are you really-really sure?', true)) {
			$query = Yii::$app->db->createCommand("DELETE FROM document");		
			$count = $query->execute();
			
			$module = Yii::$app->getModule('docManager');
			if ($path = trim($module->getWorkingPath())) {
				system('rm -rf ' . $path . '/*');
			}
			
			if ($path = trim($module->getSavedPath())) {
				system('rm -rf ' . $path . '/*');
			}
			
			echo "Deleted {$count} rows\n";
		}
	}
}
